/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include <gnome.h>
#include <libgnome/libgnome.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-stock.h>
#include "pan.h"

#include "xpm/book_open.xpm"
#include "xpm/book_closed.xpm"
#include "xpm/mini_page.xpm"
#include "xpm/greencheck.xpm"
#include "xpm/bluecheck.xpm"
#include "xpm/x.xpm"
#include "xpm/binary.xpm"
#include "xpm/binary_complete.xpm"
#include "xpm/binary_incomplete.xpm"

// Global Variables
static GnomePixmap *articlelist_closed_pixmap;
static GnomePixmap *articlelist_open_pixmap;
static GnomePixmap *decode_queue_pixmap;
static GnomePixmap *download_queue_pixmap;
static GnomePixmap *article_downloaded_pixmap;
static GnomePixmap *mp_complete_pixmap;
static GnomePixmap *mp_incomplete_pixmap;
static GdkColormap *cmap;
static GdkColor color;
static GdkColor black;

article_data *current_article = NULL;
GtkWidget *article_clist_menu;
static GtkWidget *find_dialog;

void articlelist_update_node(article_data * adata)
{
	GtkCTreeNode *node = NULL;

	g_assert(adata && adata->message_id);

	if (adata->node) {
		node = adata->node;
	} else {
		node =
			gtk_ctree_find_by_row_data(GTK_CTREE(Pan.article_ctree), NULL, adata);
	}

	if (node) {
		if (adata->state & STATE_DOWNLOAD_QUEUED) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
									adata->subject, 5,
									download_queue_pixmap->pixmap,
									download_queue_pixmap->mask,
									download_queue_pixmap->pixmap,
									download_queue_pixmap->mask,
									GTK_CTREE_ROW(node)->is_leaf,
									GTK_CTREE_ROW(node)->expanded);

		} else if (adata->state & STATE_DECODE_QUEUED) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
									adata->subject, 5,
									decode_queue_pixmap->pixmap,
									decode_queue_pixmap->mask,
									decode_queue_pixmap->pixmap,
									decode_queue_pixmap->mask,
									GTK_CTREE_ROW(node)->is_leaf,
									GTK_CTREE_ROW(node)->expanded);
		} else if (adata->state & STATE_ERROR) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
									adata->subject, 5,
									Pan.error_pixmap->pixmap,
									Pan.error_pixmap->mask,
									Pan.error_pixmap->pixmap,
									Pan.error_pixmap->mask,
									GTK_CTREE_ROW(node)->is_leaf,
									GTK_CTREE_ROW(node)->expanded);
		} else if (adata->state & STATE_DECODED) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
									adata->subject, 5,
									Pan.binary_decoded_pixmap->pixmap,
									Pan.binary_decoded_pixmap->mask,
									Pan.binary_decoded_pixmap->pixmap,
									Pan.binary_decoded_pixmap->mask,
									GTK_CTREE_ROW(node)->is_leaf,
									GTK_CTREE_ROW(node)->expanded);
	 		gtk_ctree_node_set_foreground (GTK_CTREE(Pan.article_ctree), node, &black);
		} else if (adata->state & STATE_READ) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
									adata->subject, 5,
									Pan.article_read_pixmap->pixmap,
									Pan.article_read_pixmap->mask,
									Pan.article_read_pixmap->pixmap,
									Pan.article_read_pixmap->mask,
									GTK_CTREE_ROW(node)->is_leaf,
									GTK_CTREE_ROW(node)->expanded);
	    	gtk_ctree_node_set_foreground   (GTK_CTREE(Pan.article_ctree), node, &black);
		} else {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
									adata->subject, 5, NULL, NULL, NULL,
									NULL, GTK_CTREE_ROW(node)->is_leaf,
									GTK_CTREE_ROW(node)->expanded);
	    	gtk_ctree_node_set_foreground(GTK_CTREE(Pan.article_ctree), node, &color);
		}
		/* gtk_widget_queue_draw(GTK_WIDGET(Pan.article_ctree)); */
	}
}

/**********************************************************
 * The following stuff implements the "find" functionality
 **********************************************************/

static GtkCTreeNode *next_pos = NULL;

void find_dialog_destroy_cb(void)
{
	gtk_widget_destroy(find_dialog);
	find_dialog = NULL;
}

int find_compare_func(article_data *adata, char *p)
{
	if (adata && adata->subject) {
		if (strstr(adata->subject, p)) {
			return (0);
		}
	}
	return (-1);
}

int find_article(char *p)
{
	GtkCTreeNode *node, *n;
	static char *find_text = NULL;

	if (p) {
		g_free(find_text);
		find_text = p;
	}

	if (find_text) {
		node = gtk_ctree_find_by_row_data_custom(GTK_CTREE(Pan.article_ctree), next_pos, find_text, 
					(GCompareFunc)find_compare_func);
		if (node) {
			gtk_clist_freeze(GTK_CLIST(Pan.article_ctree));
			gtk_ctree_unselect_recursive(GTK_CTREE(Pan.article_ctree), NULL);
			gtk_ctree_select(GTK_CTREE(Pan.article_ctree), node);
			/* Make sure the found node is not in a collapsed branch */
			for (n = GTK_CTREE_ROW(node)->parent; n; n = GTK_CTREE_ROW(n)->parent)
				gtk_ctree_expand(GTK_CTREE(Pan.article_ctree), n);
			gtk_ctree_node_moveto(GTK_CTREE(Pan.article_ctree), node, 0, 0, 0);
			gtk_clist_thaw(GTK_CLIST(Pan.article_ctree));
			
			/* Find next node, paying attention to recursion */
			if (GTK_CTREE_ROW(node)->children) {
				next_pos = GTK_CTREE_ROW(node)->children;
			} else {
				for (n = node; n; n = GTK_CTREE_ROW(n)->parent) {
					if (GTK_CTREE_ROW(n)->sibling) {
						next_pos = GTK_CTREE_ROW(n)->sibling;
						break;
					}
				}
			}
			/***/
		} else {
			next_pos = NULL;
		}
	}
}

void find_dialog_clicked_cb(GnomeDialog *dialog, int button, GtkWidget *find_entry)
{
	char *p;

	switch (button) {
	case 0:					/* OK button */
		p = g_strdup(gtk_entry_get_text(GTK_ENTRY(find_entry)));
		if (strlen(p) > 0) {
			find_article(p);
		}
	default:				/* Cancel Button */
		find_dialog_destroy_cb();
		break;
	}
}

void articlelist_find_next_cb(void)
{
	find_article(NULL);
}

void articlelist_find_text_cb(void)
{
	GtkWidget *hbox, *label, *find_entry;

	/* There can be only one! (find window) */
	if (find_dialog) {
		gdk_window_raise(GTK_WIDGET(find_dialog)->window);
		return;
	}
	
	find_dialog = gnome_dialog_new("Find Text", GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);

	hbox = gtk_hbox_new(FALSE, 0);
	label = gtk_label_new(_("Find:"));
	find_entry = gtk_entry_new();	
	
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
	gtk_box_pack_end(GTK_BOX(hbox), find_entry, TRUE, TRUE, 5);
	
	gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(find_dialog)->vbox), hbox);
	
	gtk_signal_connect(GTK_OBJECT(find_dialog), "clicked", GTK_SIGNAL_FUNC(find_dialog_clicked_cb), find_entry);
	gtk_signal_connect(GTK_OBJECT(find_dialog), "destroy", GTK_SIGNAL_FUNC(find_dialog_destroy_cb), NULL);
	gtk_widget_show_all(GTK_WIDGET(find_dialog));
}

void articlelist_get_all(void)
{
	if (Pan.active_group)
		queue_add_articlelist_download(Pan.active_group,
									   QUEUE_ARTICLELIST_DOWNLOAD_ALL |
									   QUEUE_FOREGROUND);
}


//
//
//
void articlelist_get_new(void)
{
	if (Pan.active_group)
		queue_add_articlelist_download(Pan.active_group,
									   QUEUE_ARTICLELIST_DOWNLOAD_NEW |
									   QUEUE_FOREGROUND);
}


//--------------------------------------------------------------------
// free an article list (and all threads)
//--------------------------------------------------------------------
void articlelist_free(GSList * alist)
{
	GSList *p;
	article_data *adata;

	for (p = alist; p != NULL; p = p->next) {
		g_assert(p->data);
		adata = p->data;

		if (adata->threads)
			articlelist_free(adata->threads);
		article_free(adata);
		p->data = NULL;
	}
	g_slist_free(alist);
}


void articlelist_select_all(void)
{
	gtk_ctree_select_recursive(GTK_CTREE(Pan.article_ctree), NULL);
}

//--------------------------------------------------------------------
// process the selected articles
//--------------------------------------------------------------------
// FIXME: is type going to be anything other than 2?
void process_selections(int type)
{
	GList *p;
	article_data *adata;

	for (p = GTK_CLIST(Pan.article_ctree)->selection; p; p = p->next) {
		adata = (article_data *) gtk_ctree_node_get_row_data(GTK_CTREE(Pan.article_ctree), p->data);
		if (adata) {
			if (type == 2) {
				if (adata->parts == 1)
					queue_add_article_download(adata, QUEUE_ARTICLE_DECODE | QUEUE_BACKGROUND);
				else if ((adata->parts >= 2) && (adata->threads))
					queue_add_multipart_download(adata, QUEUE_MULTIPART_DECODE | QUEUE_BACKGROUND);
				else
					continue;
				adata->state |= STATE_DECODE_QUEUED;
				/* Calling article_update here is too slow */
				articlelist_update_node(adata);
			}
		}
	}
	gtk_ctree_unselect_recursive(GTK_CTREE(Pan.article_ctree), NULL);
}

void download_selected_articles(void)
{
	process_selections(0);
}

void decode_selected_articles(void)
{
	process_selections(2);
}

GtkCTreeNode *add_article_to_ctree(GtkCTreeNode * parent, article_data * adata)
{
	char *text[4];
	GtkCTreeNode *node;

	if (adata->subject != NULL)
		text[0] = adata->subject;
	text[1] = g_strdup_printf("%d", adata->linecount);
	if (adata->date != 0)
		text[2] = g_strdup(ctime((const time_t *) &adata->date));
	if (adata->author != NULL)
		text[3] = adata->author;

	if (adata->state & STATE_ERROR) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
								  NULL, text, 5, Pan.error_pixmap->pixmap,
								  Pan.error_pixmap->mask,
								  Pan.error_pixmap->pixmap,
								  Pan.error_pixmap->mask, FALSE, FALSE);
	} else if (adata->state & STATE_DECODED) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
								  NULL, text, 5,
								  Pan.binary_decoded_pixmap->pixmap,
								  Pan.binary_decoded_pixmap->mask,
								  Pan.binary_decoded_pixmap->pixmap,
								  Pan.binary_decoded_pixmap->mask, FALSE,
								  FALSE);
		gtk_ctree_node_set_foreground (GTK_CTREE(Pan.article_ctree), node, &black);
	} else if (adata->state & STATE_READ) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
								  NULL, text, 5,
								  Pan.article_read_pixmap->pixmap,
								  Pan.article_read_pixmap->mask,
								  Pan.article_read_pixmap->pixmap,
								  Pan.article_read_pixmap->mask, FALSE,
								  FALSE);
		gtk_ctree_node_set_foreground (GTK_CTREE(Pan.article_ctree), node, &black);
	} else if (adata->state & STATE_DOWNLOAD_QUEUED) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
								  NULL, text, 5,
								  download_queue_pixmap->pixmap,
								  download_queue_pixmap->mask,
								  download_queue_pixmap->pixmap,
								  download_queue_pixmap->mask, FALSE,
								  FALSE);
	} else if (adata->state & STATE_DECODE_QUEUED) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
								  NULL, text, 5,
								  decode_queue_pixmap->pixmap,
								  decode_queue_pixmap->mask,
								  decode_queue_pixmap->pixmap,
								  decode_queue_pixmap->mask, FALSE, FALSE);
	} else if (adata->state & STATE_MULTIPART_ALL) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
								  NULL, text, 5,
								  mp_complete_pixmap->pixmap,
								  mp_complete_pixmap->mask,
								  mp_complete_pixmap->pixmap,
								  mp_complete_pixmap->mask, FALSE, FALSE);
	    gtk_ctree_node_set_foreground(GTK_CTREE(Pan.article_ctree), node, &color);
	} else if (adata->state & STATE_MULTIPART_SOME) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
								  NULL, text, 5,
								  mp_incomplete_pixmap->pixmap,
								  mp_incomplete_pixmap->mask,
								  mp_incomplete_pixmap->pixmap,
								  mp_incomplete_pixmap->mask, FALSE,
								  FALSE);
	    gtk_ctree_node_set_foreground(GTK_CTREE(Pan.article_ctree), node, &color);
	} else {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
								  NULL, text, 5, NULL, NULL, NULL, NULL,
								  FALSE, FALSE);
		gtk_ctree_node_set_foreground (GTK_CTREE(Pan.article_ctree), node, &color);
	}
	gtk_ctree_node_set_row_data(GTK_CTREE(Pan.article_ctree), node,
								(article_data *) adata);
	adata->node = node;
	return node;
}

static int build_article_ctree_recursive(GtkCTreeNode * parent,
										 GSList * list)
{
	GSList *pos;
	article_data *data;
	GtkCTreeNode *sibling;
	int count = 0;
	pos = list;

	while (pos) {
		data = (article_data *) pos->data;
		pos = pos->next;
		sibling = add_article_to_ctree(parent, data);
		++count;
		if (data->threads)
			count =
				count + build_article_ctree_recursive(sibling,
													  data->threads);
	}
	return (count);
}

void article_ctree_clean(void)
{
	gtk_clist_clear(GTK_CLIST(Pan.article_ctree));
	if (Pan.active_group) {
		clean_group(Pan.active_group);
		Pan.active_group = NULL;
	}
}

void articlelist_update(group_data * gdata)
{
	char *text[4];
	GtkCTreeNode *parent;

	if (gdata == NULL)
		return;

	text[0] = g_strdup(gdata->name);
	text[1] = text[2] = text[3] = NULL;

	gui_set_status(_("Updating Article Display"));
	pan_lock();
	gtk_clist_clear(GTK_CLIST(Pan.article_ctree));
	gtk_clist_freeze(GTK_CLIST(Pan.article_ctree));
	parent = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), NULL, NULL,
							  text, 5, articlelist_closed_pixmap->pixmap,
							  articlelist_closed_pixmap->mask,
							  articlelist_open_pixmap->pixmap,
							  articlelist_open_pixmap->mask, FALSE, TRUE);

	build_article_ctree_recursive(parent, gdata->articlelist);

	gtk_clist_thaw(GTK_CLIST(Pan.article_ctree));

	if (Pan.active_group && (Pan.active_group != gdata))
		clean_group(Pan.active_group);

	Pan.active_group = gdata;
	pan_unlock();
}

int multipart_sortfunc(article_data * data1, article_data * data2)
{
	if (data1->part > data2->part)
		return (1);
	else
		return (-1);
}

int mp_strcmp(char *mp, char *ph)
{
	int i = 0;
	int j = 0;

	while ((mp[i] != '\0') && (ph[j] != '\0')) {
		if (((mp[i] == '(') && (ph[j] == '(')) ||
			((mp[i] == '[') && (ph[j] == '['))) {
			++j;
			++i;
			if (isdigit(ph[j]) && isdigit(mp[i])) {
				for (; isdigit(ph[j]); ++j);
				for (; isdigit(mp[i]); ++i);
			}
		} else if (mp[i] == ph[j]) {
			i++;
			j++;
		} else
			return 1;
	}
	return 0;
}

GSList *thread_article_by_multipart(GSList * list, article_data * adata)
{
	GSList *p;
	article_data *data;

	for (p = list; p; p = p->next) {
		data = p->data;
		if ((data->parts >= 2) && (data->parts == adata->parts) && (data->part < adata->part)) {
			if (!mp_strcmp(data->subject, adata->subject)) {
				list = g_slist_remove(list, adata);
				if (adata->threads) {
					data->threads = g_slist_concat(data->threads, adata->threads);
					data->nthreads = data->nthreads + adata->nthreads;
					adata->threads = NULL;
					adata->nthreads = 0;
				}
				data->threads = g_slist_prepend(data->threads, adata);
				data->nthreads++;
				return list;
			}
		}
	}
	return list;
}

GSList *thread_list_by_multipart(GSList * list, group_data * gdata)
{
	article_data *adata;
	GSList *p;
	gfloat count, total;

	g_assert(list);

	count = 0;
	total = gdata->articles * 3;

	p = list;
	while (p) {
		gui_set_progress(count++ / total);
		adata = (article_data *) p->data;
		p = p->next;
		if (adata->parts >= 2)
			/* Found a Multipart, thread it */
			list = thread_article_by_multipart(list, adata);
	}

	for (p = list; p; p = p->next) {
		adata = (article_data *) p->data;
		if (adata->parts >= 2)
			if ((adata->nthreads == (adata->parts - 1)) || (adata->nthreads == adata->parts)) {
				adata->threads = g_slist_sort(adata->threads, (GCompareFunc)multipart_sortfunc);
				adata->state = STATE_MULTIPART_ALL;
			} else
				adata->state = STATE_MULTIPART_SOME;
	}
	return list;
}

int thread_article_by_reference(GSList * list, article_data * data)
{
	GSList *pos;
	article_data *adata;

	pos = list;

	while (pos) {
		adata = (article_data *) pos->data;
		pos = pos->next;
		if (adata->references
			&& (strcmp(data->message_id, adata->message_id))) {
			if (!strcmp(strrchr(data->references, '<'), adata->message_id)) {
				/* we found our home! Thread it here */
				adata->threads = g_slist_prepend(adata->threads, data);
				adata->nthreads++;
				return 1;
			}
			if (adata->threads) {
				if (thread_article_by_reference(adata->threads, data))
					return 1;
			}
		}
	}							/* end while loop */

	return 0;
}

GSList *thread_thread_by_reference(GSList * list)
{
	article_data *adata;
	GSList *tmp;
	int rc;

	tmp = list;
	while (tmp) {
		adata = (article_data *) tmp->data;
		tmp = tmp->next;
		if (adata->references && adata->references[0] == '<') {
			rc = thread_article_by_reference(list, adata);
			if (rc)
				list = g_slist_remove(list, adata);
		}
	}
	return list;
}

GSList *thread_list_by_reference(GSList * list, group_data * gdata)
{
	article_data *adata;
	GSList *tmp;
	gfloat count, total;

	count = gdata->articles * 2;
	total = gdata->articles * 3;
	tmp = list;

	while (tmp) {
		gui_set_progress(count++ / total);
		adata = (article_data *) tmp->data;
		tmp = tmp->next;
		if (adata->threads)
			adata->threads = thread_thread_by_reference(adata->threads);

	}
	gui_set_progress(1);
	return list;
}

int thread_article_by_subject(GSList * list, article_data * data)
{
	GSList *tmp;
	article_data *adata, *min = NULL;

	tmp = list;

	while (tmp) {
		adata = (article_data *) tmp->data;
		tmp = tmp->next;
		if (adata != data) {
			if (!strcmp(adata->subject, data->subject + 4)) {
				/* Found  "Re: name" and "name" */
				adata->threads = g_slist_prepend(adata->threads, data);
				adata->nthreads++;
				return 1;
			} else if (!strcasecmp(adata->subject, data->subject)) {
				if (adata->threads) {
					adata->threads = g_slist_prepend(adata->threads, data);
					adata->nthreads++;
					return 1;
				} else {
					if (min == NULL)
						min = (adata->date < data->date) ? adata : NULL;
					else if (adata->date < min->date)
						min = adata;
				}
			}
		}
	}
	if (min) {
		min->threads = g_slist_prepend(min->threads, data);
		min->nthreads++;
		return 1;
	} else
		return 0;
}

GSList *thread_list_by_subject(GSList * list, group_data * gdata)
{
	article_data *adata;
	GSList *pos, *last;
	int rc;
	gfloat count, total;

	g_assert(list);

	count = gdata->articles;
	total = gdata->articles * 3;

	pos = list;

	while (pos) {
		gui_set_progress(count++ / total);
		adata = (article_data *) pos->data;
		last = pos;
		pos = pos->next;
		if (!strncasecmp("re:", adata->subject, 3)) {
			rc = thread_article_by_subject(list, adata);
			if (rc) {
				list = g_slist_remove(list, adata);
			}
		}
	}
	return list;
}


//
//
void article_ctree_button_press(GtkWidget * widget,
								GdkEventButton * bevent)
{
	int row;
	int column;
	article_data *adata;

	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
									 "button_press_event");

	if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS)) {
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
									 "button_press_event");
		gtk_clist_get_selection_info(GTK_CLIST(widget), bevent->x,
									 bevent->y, &row, &column);
		adata =
			(article_data *) gtk_clist_get_row_data(GTK_CLIST(widget),
													row);
		if (adata) {
			current_article = adata;
			if (adata->body == NULL) {
				queue_add_article_download(adata,
										   QUEUE_ARTICLE_DOWNLOAD |
										   QUEUE_FOREGROUND);
				adata->state |= STATE_DOWNLOAD_QUEUED;
				articlelist_update_node(adata);
			} else
				text_update(adata);
		}
	} else if (bevent->button == 3)
		gtk_menu_popup(GTK_MENU(article_clist_menu), NULL, NULL, NULL,
					   NULL, bevent->button, bevent->time);
}


/*--------------------------------------------------------------------
// right click popup menu when clicking on an article
//--------------------------------------------------------------------*/
void create_article_ctree_menu(GtkWidget *s_window)
{
	GtkWidget *tempitem;

	article_clist_menu = gtk_menu_new();

	tempitem = gtk_menu_item_new_with_label("Decode Selected");
	gtk_menu_append(GTK_MENU(article_clist_menu), tempitem);
	gtk_signal_connect_object(GTK_OBJECT(tempitem), "activate",
							  GTK_SIGNAL_FUNC(decode_selected_articles),
							  NULL);
	tempitem = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(article_clist_menu), tempitem);
	tempitem = gtk_menu_item_new_with_label("Delete Selected");
	gtk_menu_append(GTK_MENU(article_clist_menu), tempitem);
	gtk_signal_connect_object(GTK_OBJECT(tempitem), "activate",
							  GTK_SIGNAL_FUNC(article_delete),
							  NULL);
	gtk_widget_show_all(article_clist_menu);
}

int compare_subject_ascending(article_data * data1, article_data * data2)
{
	return (strcasecmp(data1->subject, data2->subject));
}

int compare_subject_descending(article_data * data1, article_data * data2)
{
	return (strcasecmp(data2->subject, data1->subject));
}

int compare_lines_ascending(article_data * data1, article_data * data2)
{
	if (data1->linecount > data2->linecount)
		return (1);
	else
		return (-1);
}

int compare_lines_descending(article_data * data1, article_data * data2)
{
	if (data2->linecount > data1->linecount)
		return (1);
	else
		return (-1);
}

int compare_author_ascending(article_data * data1, article_data * data2)
{
	return (strcasecmp(data1->author, data2->author));
}

int compare_author_descending(article_data * data1, article_data * data2)
{
	return (strcasecmp(data2->author, data1->author));
}

int compare_date_ascending(article_data * data1, article_data * data2)
{
	if (data1->date > data2->date)
		return (1);
	else
		return (-1);
}

int compare_date_descending(article_data * data1, article_data * data2)
{
	if (data2->date > data1->date)
		return (1);
	else
		return (-1);
}

void click_articlelist_column(GtkCList * clist, int n)
{
	static int subject = 1;
	static int lines = 1;
	static int date = 1;
	static int author = 1;

	if (Pan.active_group == NULL)
		return;

	switch (n) {
	case 0:					// Subject

		if (subject)
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_subject_ascending);
		else
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_subject_descending);
		subject = subject ? 0 : 1;
		break;

	case 1:					// Lines

		if (lines)
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_lines_ascending);
		else
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_lines_descending);
		lines = lines ? 0 : 1;
		break;

	case 2:					// Date

		if (date)
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_date_ascending);
		else
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_date_descending);
		date = date ? 0 : 1;
		break;

	case 3:					// Author

		if (author)
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_author_ascending);
		else
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_author_descending);
		author = author ? 0 : 1;
		break;

	default:
		g_warning("Reached default in click_articlelist_column()\n");
		return;
	}
	articlelist_update(Pan.active_group);
}


void click_articlelist_column2(GtkCList * clist, int n)
{
	static int subject = 1;
	static int lines = 1;
	static int date = 1;
	static int author = 1;

	if (Pan.active_group == NULL)
		return;

	switch (n) {
	case 0:					// Subject

		if (subject) {
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_subject_ascending);

			subject = 0;
		} else {
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_subject_descending);
			subject = 1;
		}
		break;
	case 1:					// Lines

		if (lines) {
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_lines_ascending);
			lines = 0;
		} else {
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_lines_descending);
			lines = 1;
		}
		break;
	case 2:					// Date

		if (date) {
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_date_ascending);
			date = 0;
		} else {
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_date_descending);
			date = 1;
		}
		break;
	case 3:					// Author

		if (author) {
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_author_ascending);
			author = 0;
		} else {
			Pan.active_group->articlelist =
				g_slist_sort(Pan.active_group->articlelist,
							 (GCompareFunc) compare_author_descending);
			author = 1;
		}
		break;
	default:
		g_warning("Reached default in click_articlelist_column()\n");
		return;
	}
	articlelist_update(Pan.active_group);
}


//--------------------------------------------------------------------
// generate the listing of articles, for the "Articles" tab
//--------------------------------------------------------------------
GtkWidget *create_articlelist_ctree(void)
{
	GtkWidget *s_window;

	static char *titles[] = {
		_("Subject"),
		_("Lines"),
		_("Date"),
		_("Author"),
	};

	articlelist_closed_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(book_closed_xpm);
	articlelist_open_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(book_open_xpm);
	Pan.article_read_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(mini_page_xpm);
	decode_queue_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(bluecheck_xpm);
	download_queue_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(greencheck_xpm);
	article_downloaded_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(mini_page_xpm);
	Pan.error_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(x_xpm);
	Pan.binary_decoded_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_xpm);
	mp_complete_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_complete_xpm);
	mp_incomplete_pixmap =
		(GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_incomplete_xpm);

	cmap = gdk_colormap_get_system();
	color.red = 0xffff;
	color.green = 0;
	color.blue = 0;
	black.red = 0;
	black.green = 0;
	black.blue = 0;
	if (!gdk_color_alloc(cmap, &color)) {
		g_error("couldn't allocate color");
	}
	if (!gdk_color_alloc(cmap, &black)) {
          g_error("couldn't allocate color");
	}

	Pan.article_ctree = gtk_ctree_new_with_titles(4, 0, titles);
	gtk_ctree_set_line_style(GTK_CTREE(Pan.article_ctree),
							 GTK_CTREE_LINES_DOTTED);
	gtk_widget_show(Pan.article_ctree);

	s_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(s_window),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(s_window), Pan.article_ctree);
	gtk_widget_show(s_window);

	gtk_signal_connect(GTK_OBJECT(Pan.article_ctree), "button_press_event",
					   GTK_SIGNAL_FUNC(article_ctree_button_press), NULL);

	gtk_signal_connect(GTK_OBJECT(Pan.article_ctree), "click_column",
					   GTK_SIGNAL_FUNC(click_articlelist_column), NULL);

	gtk_clist_set_selection_mode(GTK_CLIST(Pan.article_ctree),
								 GTK_SELECTION_EXTENDED);
	gtk_clist_set_column_width(GTK_CLIST(Pan.article_ctree), 0, 400);
	gtk_clist_set_column_width(GTK_CLIST(Pan.article_ctree), 1, 50);
	gtk_clist_set_column_width(GTK_CLIST(Pan.article_ctree), 2, 150);

	// Create the Popup Menu While we are here
	create_article_ctree_menu(s_window);

	return s_window;
}


//--------------------------------------------------------------------
// load article listing
//--------------------------------------------------------------------
void articlelist_load(queue_data * qdata)
{
	group_data *gdata;
	GSList *alist;

	gdata = qdata->gdata;

	/* step 1: load article headers */
	gui_set_status(_("Loading Article Headers"));
	if ((alist = db_load_articles(gdata)) == NULL) {
		pan_lock();
		choose_download_dialog(gdata);
		pan_unlock();
		goto articlelist_load_cleanup;
	}
	grouplist_update_row(gdata);
	gui_set_info_va(_("Loaded %d article headers"), gdata->articles);

	/* step 2: thread article list */
	gui_set_status(_("Threading Article List"));

	alist = thread_list_by_multipart(alist, gdata);
	alist = thread_list_by_subject(alist, gdata);
	alist = thread_list_by_reference(alist, gdata);
	gdata->articlelist = alist;

	/* step 3: display article list */
	gui_set_status(_("Displaying article list"));
	articlelist_update((group_data *) gdata);

  articlelist_load_cleanup:
	g_free(qdata);
	gui_clean();
}
